<?php 
/**
 * @file
 * This file handles all the conditional actions stuff
 *
 */
 
 
/**
 * Implements hook_ca_predicate().
 */
function uc_event_registration_ca_predicate() {  
 
  $configurations['uc_event_registration_update_components'] = array(
    '#title' => t("Update Paid Event webform components"),
    '#description' => t("Update Paid Event webform components with data from the order, such as the order status."),
    '#class' => 'Event Registration',
    '#trigger' => 'uc_order_status_update',
    '#status' => 1,
    '#conditions' => array(
      '#operator' => 'AND',
      '#conditions' => array(
        array(
          '#name' => 'uc_cart_condition_product_class',
          '#title' => t("Order contains a Paid Event"),
          '#argument_map' => array(
            'order' => 'updated_order',
          ),
          '#settings' => array(
            'negate' => FALSE,
            'class' => 'paidevent', 
          ),
        ),
      ),
    ),
    '#actions' => array(
      array(
        '#name' => 'uc_event_registration_update_components',
        '#title' => t('Update Paid Event webform components'),
        '#argument_map' => array(
          'order' => 'updated_order', 
        ),
      ),
    ),
  );
  
  $configurations['uc_event_registration_send_webform_emails'] = array(
    '#title' => t("Send Paid Event 'late' emails"),
    '#description' => t("For Paid Events send any emails configured for 'late' sending."),
    '#class' => 'Event Registration',
    '#trigger' => 'uc_checkout_complete',
    '#status' => 1,
    '#conditions' => array(
      '#operator' => 'AND',
      '#conditions' => array(
        array(
          '#name' => 'uc_cart_condition_product_class',
          '#title' => t("Order contains a Paid Event"),
          '#argument_map' => array(
            'order' => 'order',
          ),
          '#settings' => array(
            'negate' => FALSE,
            'class' => 'paidevent', 
          ),
        ),
      ),
    ),
    '#actions' => array(
      array(
        '#name' => 'uc_event_registration_send_webform_emails',
        '#title' => t("Send webform emails"),
        '#argument_map' => array(
          'order' => 'order', 
        ),
      ),
    ),
  );
  return $configurations;
}

/**
 * Implementation of hook_ca_action().
 *
 * Provides two actions:
 *   Mark an event registration webform as paid after checkout
 *   Send any emails configured to send on checkout completion
 */
function uc_event_registration_ca_action() {
  $order_arg = array(
    '#entity' => 'uc_order',
    '#title' => t('Order'),
  );

  $actions['uc_event_registration_update_components'] = array(
    '#title' => t('Update Paid Event webform components'),
    '#category' => t('Event Registration'),
    '#callback' => 'uc_event_registration_update_components',
    '#arguments' => array(
      'order' => $order_arg,
    ),
  );
  
  $actions['uc_event_registration_send_webform_emails'] = array(
    '#title' => t('Send webform emails'),
    '#category' => t('Event Registration'),
    '#callback' => 'uc_event_registration_send_webform_emails',
    '#arguments' => array(
      'order' => $order_arg,
    ),
  );
  
  return $actions;
}
  
/**
 * CA action: Update Paid Event webform components with fresh data from the
 * order.
 */
function uc_event_registration_update_components(&$order, $settings) {
  
  module_load_include('inc', 'webform', 'includes/webform.submissions');
  $cids = array();
  
  $payment_status = (uc_payment_balance($order) > 0) ? 'unpaid' : 'paid';

  // Loop through the order and update appropriate components for all paidevent
  // products.
  foreach ($order->products as $product) {
    if (isset($product->data['uc_event_registration']['sid'])) {
      $nid = $product->nid;
      $sid = $product->data['uc_event_registration']['sid'];
      
      $submission = webform_get_submission($nid, $sid);
      if ($submission) {
        // It's a valid webform and submission
      
        $new_values = array(
          'uc_event_payment_status' => $payment_status,
          'uc_event_quantity' => $product->qty,
          'uc_event_order_status' => $order->order_status,
        );
      
        $node = node_load($nid);
        
        if (!isset($cids[$nid])) {
          // Map component form keys to their CIDs.
          foreach ($node->webform['components'] as $component) {
            if (isset($new_values[$component['form_key']])) {
              $cids[$nid][$component['form_key']] = $component['cid'];
            }
          }
        }
      
        // Modify $submission to match the data in $new_values.
        foreach ($new_values as $form_key => $value) {
          if (isset($cids[$nid][$form_key])) {
            $cid = $cids[$nid][$form_key];
            $submission->data[$cid]['value'][0] = $value;
          }
        }
        
        // Save the submission.
        webform_submission_update($node, $submission);
      }
    }
  }
}

/** 
 * CA action: Send all late emails associated with paidevents in this order.
 */
function uc_event_registration_send_webform_emails(&$order, $settings){
  // Include the webform submissions file which contains all email functions
  include_once drupal_get_path('module', 'webform') .'/includes/webform.submissions.inc';
  foreach ($order->products as $theproduct) {
    // load the node object which is used by the webform submission functions
    $node = node_load(array("nid" => $theproduct->nid));
    $sid = $theproduct->data['uc_event_registration']['sid'];
    $submission = webform_get_submission($theproduct->nid, $sid, TRUE);
    
    // Process node to remove emails that have already been sent
    _uc_event_registration_process_emails($node, UC_EVENT_REGISTRATION_EMAIL_ON_CHECKOUT);
    
    // now send the email
    webform_submission_send_mail($node, $submission);
  }
}
